// ScribbleDoc.cpp : implementation of the CScribbleDoc class
//

#include "stdafx.h"
#include <afxdlgs.h>

#include "Scribble.h"

#include "ScribbleDoc.h"
#include "PenWidthsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc

IMPLEMENT_DYNCREATE(CScribbleDoc, CDocument)

BEGIN_MESSAGE_MAP(CScribbleDoc, CDocument)
	//{{AFX_MSG_MAP(CScribbleDoc)
	ON_COMMAND(ID_EDIT_CLEARALL, OnEditClearall)
	ON_COMMAND(ID_PEN_THICK_OR_THIN, OnPenThickOrThin)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CLEARALL, OnUpdateEditClearall)
	ON_UPDATE_COMMAND_UI(ID_PEN_THICK_OR_THIN, OnUpdatePenThickOrThin)
	ON_COMMAND(ID_PEN_PENWIDTHS, OnPenPenwidths)
	ON_COMMAND(ID_PEN_COLOR, OnPenColor)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_FILE_SEND_MAIL, OnFileSendMail)
	ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL, OnUpdateFileSendMail)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc construction/destruction

CScribbleDoc::CScribbleDoc()
{
	// TODO: add one-time construction code here
}

CScribbleDoc::~CScribbleDoc()
{
}

BOOL CScribbleDoc::OnNewDocument()
{
	if(!CDocument::OnNewDocument())
		return FALSE;
	InitDocument();
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc serialization

void CScribbleDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
 m_strokeList.Serialize( ar );
}

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc diagnostics

#ifdef _DEBUG
void CScribbleDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CScribbleDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CScribbleDoc commands

CStroke * CScribbleDoc::NewStroke()
{
CStroke* pStrokeItem = new CStroke(m_nPenWidth,rCur,gCur,bCur);
	m_strokeList.AddTail( pStrokeItem );
	SetModifiedFlag( );	// Mark document as modified 
					// to confirm File Close.
	return pStrokeItem;
}

void CScribbleDoc::InitDocument()
{
	m_bThickPen = FALSE;
	m_nThinWidth = 2;     // Default thin pen is 2 pixels wide
	m_nThickWidth = 5;    // Default thick pen is 5 pixels wide

	rCur=0;			// Default color is a dark blue
	gCur=0;
	bCur=64;

	ReplacePen();   // Initialize pen according to current width
}

BOOL CScribbleDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if(!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;
	InitDocument();
	return TRUE;
}

void CScribbleDoc::DeleteContents() 
{
	while( !m_strokeList.IsEmpty( ) )
	{
	delete m_strokeList.RemoveHead( );
	}
	
	CDocument::DeleteContents();
}


//////////////////////////////////////////////////////
// CStroke
IMPLEMENT_SERIAL( CStroke, CObject, 1 )
CStroke::CStroke()
{
// This empty constructor should be used by serialization only
}

CStroke::CStroke(UINT nPenWidth, BYTE pr,BYTE pg,BYTE pb)
{
	m_nPenWidth = nPenWidth;
	r=pr;
	g=pg;
	b=pb;
}

  
void CStroke::Serialize( CArchive& ar )
{
	if( ar.IsStoring( ) )
	{
		ar << (WORD)m_nPenWidth;
		m_pointArray.Serialize( ar );
	}
	else
	{
		WORD w;
	    ar >> w;
		m_nPenWidth = w;
		m_pointArray.Serialize( ar );
	}
}

BOOL CStroke::DrawStroke( CDC* pDC )
{
	CPen penStroke;
	if( !penStroke.CreatePen(PS_SOLID, m_nPenWidth, RGB(r,g,b))) 
		return FALSE;
	CPen* pOldPen = pDC->SelectObject( &penStroke );
	pDC->MoveTo( m_pointArray[0] );
	for( int i=1; i < m_pointArray.GetSize(); i++ )
	{
		pDC->LineTo( m_pointArray[i] );
	}
	pDC->SelectObject( pOldPen );
	return TRUE;
}

 

void CScribbleDoc::OnEditClearall() 
{
	DeleteContents( );
	SetModifiedFlag(FALSE);
	UpdateAllViews( NULL );
}

void CScribbleDoc::OnPenThickOrThin() 
{
	// Toggle the state of the pen between thin and thick.
m_bThickPen = !m_bThickPen;

// Change the current pen to reflect the new width.
ReplacePen( );
}

void CScribbleDoc::ReplacePen()
{
m_nPenWidth = m_bThickPen ? m_nThickWidth : m_nThinWidth;
	// Change the current pen to reflect the new width.
	m_penCur.DeleteObject( );
	m_penCur.CreatePen( PS_SOLID, m_nPenWidth, RGB(rCur,gCur,bCur) );
}

void CScribbleDoc::OnUpdateEditClearall(CCmdUI* pCmdUI) 
{
	// Enable the user-interface object (menu item or tool- 
// bar button) if the document is non-empty, i.e., has 
// at least one stroke.
 pCmdUI->Enable( !m_strokeList.IsEmpty( ) );
}

void CScribbleDoc::OnUpdatePenThickOrThin(CCmdUI* pCmdUI) 
{
	// Add check mark to Pen Thick Line menu item if the current
	// pen width is "thick."
	pCmdUI->SetCheck( m_bThickPen );
}

void CScribbleDoc::OnPenPenwidths() 
{
	CPenWidthsDlg dlg;
	// Initialize dialog data
	dlg.m_nThinWidth = m_nThinWidth;
	dlg.m_nThickWidth = m_nThickWidth;

	// Invoke the dialog box
	if (dlg.DoModal() == IDOK)
	{
		// retrieve the dialog data
		m_nThinWidth = dlg.m_nThinWidth;
		m_nThickWidth = dlg.m_nThickWidth;

		// Update the pen used by views when drawing new strokes
		// to reflect the new pen widths for "thick" and "thin".
		ReplacePen();
	}
}

void CScribbleDoc::OnPenColor() 
{
	CColorDialog MyColDlg;
	MyColDlg.DoModal();
	rCur=GetRValue(MyColDlg.GetColor());
	gCur=GetGValue(MyColDlg.GetColor());
	bCur=GetBValue(MyColDlg.GetColor());
	ReplacePen();
}
